-module(ssh_manager).

-behaviour(gen_server).

%% API
-export([start_link/0]).

%% gen_server callbacks
-export([
         init/1,
         handle_call/3,
         handle_cast/2,
         handle_info/2,
         terminate/2,
         code_change/3
        ]).

-define(SERVER, ?MODULE).
-define(HIBERNATE_TIMEOUT, 1000).

-record(state,
        {
          ssh_daemon_ref :: pid(),
          ssh_port :: integer(),
          ssh_dir :: nonempty_string(),
          hibernate_timeout = ?HIBERNATE_TIMEOUT :: integer()
        }).

start_link() ->
    gen_server:start_link({local, ?SERVER}, ?MODULE, [], []).

init([]) ->
    process_flag(trap_exit, true),
    case start_ssh_daemon() of
        {ok, DaemonRef, Port, Ssh} ->
            {ok, #state{ssh_daemon_ref = DaemonRef, ssh_port = Port, ssh_dir = Ssh, hibernate_timeout = ?HIBERNATE_TIMEOUT}, ?HIBERNATE_TIMEOUT};
        error ->
            {ok, #state{ssh_daemon_ref = undefined, hibernate_timeout = ?HIBERNATE_TIMEOUT}, ?HIBERNATE_TIMEOUT}
    end.

handle_call(_Request, _From, #state{hibernate_timeout = HibernateTimeout} = State) ->
    Reply = ok,
    {reply, Reply, State, HibernateTimeout}.

handle_cast(start, #state{ssh_daemon_ref = undefined, hibernate_timeout = HibernateTimeout}) ->
    case start_ssh_daemon() of
        {ok, DaemonRef, Port, Ssh} ->
            {noreply, #state{ssh_daemon_ref = DaemonRef, ssh_port = Port, ssh_dir = Ssh}, HibernateTimeout};
        error ->
            {noreply, #state{ssh_daemon_ref = undefined}, HibernateTimeout}
    end;

handle_cast(stop, #state{ssh_daemon_ref = DaemonRef, hibernate_timeout = HibernateTimeout}) ->
    ssh:stop_daemon(DaemonRef),
    {noreply, #state{ssh_daemon_ref = undefined}, HibernateTimeout};

handle_cast(_Msg, #state{hibernate_timeout = HibernateTimeout} = State) ->
    {noreply, State, HibernateTimeout}.


handle_info(_Info, #state{hibernate_timeout = HibernateTimeout} = State) ->
    {noreply, State, HibernateTimeout}.

terminate(_Reason, #state{ssh_daemon_ref = DaemonRef}) ->
    case DaemonRef of
        undefined ->
            ok;
        _ ->
            ssh:stop_daemon(DaemonRef)
    end.

code_change(_OldVsn, #state{hibernate_timeout = HibernateTimeout} = State, _Extra) ->
    {ok, State, HibernateTimeout}.

start_ssh_daemon() ->
    crypto:start(),
    ssh:start(),
    Ssh = "./.ssh",
            case gen_tcp:listen(7777, [{reuseaddr, true}]) of
                {ok, Socket} ->
                    gen_tcp:close(Socket),
                    try
                        case ssh:daemon(7777, [
                                               {system_dir, Ssh},
                                               {shell, fun ssh_shell:start/2},
                                               {pwdfun, fun ssh_shell:validate_usr/2}
                                              ]) of
                            {ok, Daemon} when is_pid(Daemon) ->
                                {ok, Daemon, 7777, Ssh};
                            Other ->
                                error
                        end
                    catch
                        _:Error ->
			    ST = erlang:get_stacktrace(),
                            error
                    end;
                {error, Error} ->
                    error
            end.